#pragma once

#include <vector>
#include "WMAllocator.h"
#include "Parameter.h"
#include "PacketTypeSystem.h"
#include "WMPacket.h"
#include "Ports.h"

namespace WorldMachine {

	/** DeviceVersion contains the two most common things that change between versions of Devices; the ParameterSet and the PortSet.

	It derives from each so that you can freely use a DeviceVersion as you would directly either of its components. When the DeviceVersionController changes the device version,
	it will automatically set the ports and parameters appropriately for each device version.
	
	*/
	class DeviceVersion
	{
	public:
		DeviceVersion();
		~DeviceVersion();

		bool Load(std::istream &in);
		bool Save(std::ostream &out);

		void UpdateFromVersion(DeviceVersion &other);
		
		void setLabel(const char*label);
		const char*getLabel() const;

		void setWhatsNew(WMString const&str);
		WMString const&whatsnew() const;

		bool isNew() const;

		bool hasPortData() const;

		// Parameter related functions
		void AddParam(Parameter const&p);
		void AddParam(ParameterSet const&merging);

		bool AddDependence(const char *dependsOwner, const char *dependsOn, DEPENDS_TYPE type, VAL dependsval, DEPENDS_FUNC func = NULL);
		bool AddDependence(const char *dependsOwner, DEPENDS_FUNC func = NULL);
		Parameter *GetParam(const char *name);
		Parameter *GetParam(int index);

		ParameterSet &getParamSet();

		// Port related functions
		void SetLinks(int in, int out, WorldMachine::PacketType packet_type = WorldMachine::Packet::hfield, int common_flags = 0);

		/// Set a port's definition
		PortDefinition *in(int slot);

		/// Set a port's definition
		PortDefinition *out(int slot);

		int numInputs();
		int numOutputs();


		int version_number;
		char version_label[PARAM_NAME_SIZE];
		bool used;
		WMString sWhatsNew;

	private:
		bool bhasPortData;
		std::vector<PortDefinition> ind, outd;
		ParameterSet params;

		// In 3026 we made DeviceVersions also contain port data, and all of the below, and included it directly
		struct VersionSet_Pre3026 {
			bool used;
			int version_number;
			char name[PARAM_NAME_SIZE];
			ParameterSet params;
		};

	};
}
